

/* ***********************************************************************
                Naval Air Warfare Center / Aircraft Division              
                        Patuxent Naval Air Station                         
 ========================================================================

   File:           antenna.h                                         
                                                                        
   Classification: UNCLASSIFIED                                   
                                                                        
   Description:  This file contains the macros, structures, and classes
      needed to represent SWEG/JIMM's irregular tables
                                                                        
   Written by:	Alex Harper
*********************************************************************** */


#if !defined __ANTENNA_H
#define __ANTENNA_H

#include <stdio.h>

#include "basic_dt.h"
#include "matrix.h"

#include "swegstring.h"

#define TABLE_FREQUENCY    1
#define TABLE_ELEVATION    2
#define TABLE_AZIMUTH      3
#define TABLE_GAIN         4
#define TABLE_RCS          5
#define TABLE_ALT_XMIT     6
#define TABLE_ALT_RCVR     7
#define TABLE_2D_DIST      8
#define TABLE_POL          9

#define POL_DEFAULT       0
#define POL_HORIZ         1
#define POL_VERT          2
#define POL_LEFT_CIR      3
#define POL_RIGHT_CIR     4

#define SPEED_LIGHT	299792800.0    // meters/sec


typedef struct {
   vector3d up;			// up vector
   vector3d forward;		// forward vector
   vector3d pos;		// position
} antenna_orientation_type;


/* ***********************************************************
*********************************************************** */
class azel_slice : public dbl_llist {

   public:
      double az, el;
      double data;

      virtual ~azel_slice() {}
};


/* ***********************************************************
*********************************************************** */
class azel_manager : public dbl_llist {

   public:
      dbl_llist_manager man;

      virtual ~azel_manager() {}
};


/* ***********************************************************
   This is the base class for a level in a SWEG irregular table
*********************************************************** */
class table_type {

   public:
      dbl_llist_manager intervalman;

      virtual ~table_type() {}
      
      virtual void dump_data(char *prefix);
      virtual void extract_tdbdat(FILE *infile, char *token, int *errorflag);
      virtual void extract_data(dbl_llist_manager *man, double *az, double *el) = 0;
      virtual double query_data(double freq, double az, double el) { return 0; }
      virtual double query_data(double freq, double alt_xmit, double alt_rec, double dis_2d) { return 0; }

      virtual int query_whatami() = 0;
      virtual void query_minmax(double *data, double *az_range, double *el_range) = 0;
};


/* ***********************************************************
   This class represents an interval for a SWEG irregular table 
*********************************************************** */
class interval_type : public dbl_llist {

   public:
      double min, max;		// min/max values for the interval
      int type;

      table_type *data;		// either a sub table or data associated w/ this table

      interval_type() { data = NULL; }
      virtual ~interval_type() { if (data) delete data; }

      int query_between(double x);
};


/* ***********************************************************
   This class is used to keep the master list of all the 
   irregular tables
*********************************************************** */
class susceptibility_type : public dbl_llist {

   public:
      string_type name; 		// name of the table
      table_type *rlist;		// pointer to table data
      table_type *elist;		// pointer to table data
      
      susceptibility_type() { rlist = elist = NULL; }
      virtual ~susceptibility_type() { if (rlist) delete rlist; if (elist) delete elist; }

      void extract_tdbdat(FILE *infile, char *token, int *errorflag);
      susceptibility_type *find_table(char *x);
};


/* ***********************************************************
   This class is used to make sub-lists from the master list
   of irregular tables
*********************************************************** */
class susceptibility_link : public dbl_llist {

   public:
      susceptibility_type *list;		// pointer to a table_list in the master list
      
      susceptibility_link() { list = NULL; }
      virtual ~susceptibility_link() { }
};


/* ***********************************************************
  interval in (HZ) - frequency level of an irregular table
*********************************************************** */
class frequency_type : public table_type {

   public:
      double query_data(double freq, double az, double el);
      double query_data(double freq, double alt_xmit, double alt_rec, double dis_2d);
      int query_whatami() { return TABLE_FREQUENCY; }
      void query_minmax(double *data, double *az_range, double *el_range);
      void extract_data(dbl_llist_manager *man, double *az, double *el);
};

 
/* ***********************************************************
  azimuth level of an irregular table
*********************************************************** */
class azimuth_type : public table_type {

   public:
      double query_data(double freq, double az, double el);
      double query_data(double freq, double alt_xmit, double alt_rec, double dis_2d);
      int query_whatami() { return TABLE_AZIMUTH; }
      void query_minmax(double *data, double *az_range, double *el_range);
      void extract_data(dbl_llist_manager *man, double *az, double *el);
};


/* ***********************************************************
  elevation level of an irregular table
*********************************************************** */
class elevation_type : public table_type {

   public:
      double query_data(double freq, double az, double el);
      double query_data(double freq, double alt_xmit, double alt_rec, double dis_2d);
      int query_whatami() { return TABLE_ELEVATION; }
      void query_minmax(double *data, double *az_range, double *el_range);
      void extract_data(dbl_llist_manager *man, double *az, double *el);
};


/* ***********************************************************
*********************************************************** */
class alt_xmit_type : public table_type {

   public:
      double query_data(double freq, double az, double el);
      double query_data(double freq, double alt_xmit, double alt_rec, double dis_2d);
      int query_whatami() { return TABLE_ALT_XMIT; }
      void query_minmax(double *data, double *az_range, double *el_range);
      void extract_data(dbl_llist_manager *man, double *az, double *el);
};


/* ***********************************************************
*********************************************************** */
class alt_rcvr_type : public table_type {

   public:
      double query_data(double freq, double az, double el);
      double query_data(double freq, double alt_xmit, double alt_rec, double dis_2d);
      int query_whatami() { return TABLE_ALT_RCVR; }
      void query_minmax(double *data, double *az_range, double *el_range);
      void extract_data(dbl_llist_manager *man, double *az, double *el);
};


/* ***********************************************************
*********************************************************** */
class dist_2d_type : public table_type {

   public:
      double query_data(double freq, double az, double el);
      double query_data(double freq, double alt_xmit, double alt_rec, double dis_2d);
      int query_whatami() { return TABLE_2D_DIST; }
      void query_minmax(double *data, double *az_range, double *el_range);
      void extract_data(dbl_llist_manager *man, double *az, double *el);
};


/* ***********************************************************
  gain level of an irregular table
*********************************************************** */
class gain_type : public table_type {

   public:
      double gain;	// gain for this interval of an irregular table

      void extract_tdbdat(FILE *infile, char *token, int *errorflag);
      double query_data(double freq, double az, double el);
      double query_data(double freq, double alt_xmit, double alt_rec, double dis_2d);
      void dump_data(char *prefix);
      int query_whatami() { return TABLE_GAIN; }
      void query_minmax(double *data, double *az_range, double *el_range);
      void extract_data(dbl_llist_manager *man, double *az, double *el);
};


/* ***********************************************************
  rcs level of an irregular table
*********************************************************** */
class rcs_type : public table_type {

   public:
      double rcs;	// rcs for this interval of an irregular table

      void extract_tdbdat(FILE *infile, char *token, int *errorflag);
      double query_data(double freq, double az, double el);
      double query_data(double freq, double alt_xmit, double alt_rec, double dis_2d);
      void dump_data(char *prefix);
      int query_whatami() { return TABLE_RCS; }
      void query_minmax(double *data, double *az_range, double *el_range);
      void extract_data(dbl_llist_manager *man, double *az, double *el);
};


/* ***********************************************************
*********************************************************** */
class pol_type : public table_type {

   public:
      void extract_tdbdat(FILE *infile, char *token, int *errorflag);
      double query_data(double freq, double az, double el);
      double query_data(double freq, double alt_xmit, double alt_rec, double dis_2d);
      int query_whatami() { return TABLE_POL; }
      void query_minmax(double *data, double *az_range, double *el_range);
      void extract_data(dbl_llist_manager *man, double *az, double *el);
};


double get_scale(char *start, char *end);
void VecAng(double *bistatic, double *ux, double *uz, double *az, double *el);
void calc_azel(antenna_orientation_type *target, double *rec_pos, double *xmit_pos, double *az, double *el);

#endif
